#include "TecDummy.h"
#include <arpa/inet.h>

TecDummy::BackendRegisterer TecDummy::backendRegisterer;

/*********************************************************************************************************************/

TecDummy::BackendRegisterer::BackendRegisterer() {
  std::cout << "TecDummy::BackendRegisterer: registering backend type TecDummy" << std::endl;
  ChimeraTK::BackendFactory::getInstance().registerBackendType("TecDummy", &TecDummy::createInstance);
}

/*********************************************************************************************************************/

void TecDummy::read(uint8_t bar, uint32_t address, int32_t* data, size_t sizeInBytes) {
  DummyBackend::read(bar, address, data, sizeInBytes);
  if(bar != TecDummy_bar || address != TecDummy_address_loopDone) return;
  if(reg_loopStart != 1) return;

  // convert command into string
  // FIXME: this can be done better if the DummyRegisterRawAccessor would allow us to use the raw pointer to this...
  std::string commandString(reg_command.getNumberOfElements() * 4, 0);
  for(size_t i = 0; i < reg_command.getNumberOfElements(); ++i) {
    uint32_t word = ntohl(reg_command[i]);
    std::memcpy(&(commandString[4 * i]), &word, 4);
  }

  // process frame by frame
  constexpr size_t nFrames = 16;
  constexpr size_t frameLength = 16 * 4;
  std::string reply;
  for(size_t frame = 0; frame < nFrames; ++frame) {
    size_t start = frame * frameLength + 8;
    auto frameData = commandString.substr(start, frameLength);

    // remove everything from the 0x0D character, because only 0x00 following
    auto endOfFrame = frameData.find_first_of("\r");
    if(endOfFrame == std::string::npos) {
      reply += std::string(frameLength, '\0');
      continue;
    }
    frameData.erase(endOfFrame);

    // perform data processing
    auto frameReply = processFrame(frameData);

    // append to answer
    frameReply += "\r";
    frameReply += std::string(frameLength - frameReply.size(), '\0');
    reply += frameReply;
  }
  assert(reply.size() == nFrames * frameLength);

  // convert reply back into byte stream
  for(size_t i = 0; i < reply.length(); ++i) {
    uint32_t word;
    std::memcpy(&word, &(reply[4 * i]), 4);
    reg_command[i] = htonl(word);
  }

  reg_loopDone = 1;
}

/*********************************************************************************************************************/

std::string TecDummy::processFrame(const std::string& command) {
  // Place data processing code here. Parse data from the "command" dummy accessor and place the reply on the same
  // accessor. Just return this function, the done flag is automatically set by the caller. Remove this comment after
  // implementation.
  std::cout << "TecDummy::processFrame() command = " << command << std::endl;
  return "MY REPLY";
}

/*********************************************************************************************************************/
